define("scribe-loader", [
    'jquery',
    'scribe',
    "$dict!ep/dict",

    "scribe-plugin-toolbar",
    "scribe-plugin-sanitizer",
    "scribe-plugin-blockquote-command",
    "scribe-plugin-link-tooltip",
    "scribe-plugin-intelligent-unlink-command",
    "scribe-plugin-toolbar-dropdown",
    "scribe-plugin-heading-command",
    "scribe-plugin-paragraph-command",
    "scribe-plugin-tab-indent",
    "scribe-plugin-formatter-html-ensure-semantic-elements"
], function (
    $,
    Scribe,
    epDict,
    scribePluginToolbar,
    scribePluginSanitizer,
    scribePluginBlockquote,
    scribePluginLinkTooltip,
    scribePluginIntelligentUnlinkCommand,
    scribePluginToolbarDropdown,
    scribePluginHeadingCommand,
    scribePluginParagraphCommand,
    scribePluginTabIndent,
    scribePluginFormatterHtmlEnsureSemanticElements
) {
    'use strict';

    var translationsDialog = {
        pasteOrTypeLink: epDict.translate('PasteOrTypeLink')
    };

    return function (domEditableElement, toolbarElement) {
        var $domEditableElement = $(domEditableElement),
            scribe = new Scribe($domEditableElement[0], {
                allowBlockElements: true
            }),
            toolbarElement = toolbarElement || $domEditableElement.find('.toolbar')[0],
            scribePluginLinkTooltipTemplate;

        scribe.use(scribePluginSanitizer({
            tags: {
                p: {style: true},
                h1: {style: true},
                h2: {style: true},
                h3: {style: true},
                ul: {style: true},
                ol: {style: true},
                li: {style: true},
                blockquote: {style: true},
                strong: {},
                b: {}, // because sanitizer is called before semantic-formatter
                em: {},
                i: {}, // because sanitizer is called before semantic-formatter
                u: {},
                a: {
                    href: true
                }
            }
        }));

        scribe.use(scribePluginBlockquote());
        scribe.use(scribePluginTabIndent());

        scribe.use(scribePluginHeadingCommand(1));
        scribe.use(scribePluginHeadingCommand(2));
        scribe.use(scribePluginHeadingCommand(3));
        scribe.use(scribePluginParagraphCommand());

        scribe.use(scribePluginToolbar(toolbarElement));
        scribe.use(scribePluginToolbarDropdown(toolbarElement, {
            defaultCommand: 'p',
            selector: '.dali-plugin-text-toolbar-dropdown',
            activeSelector: '.dali-plugin-text-toolbar-dropdown-active'
        }));


        // Replace with your templating solution of choice
        scribePluginLinkTooltipTemplate = '' +
            '<div data-scribe-plugin-link-tooltip-role="arrow"></div>' +
            '<a data-scribe-plugin-link-tooltip-role="link"' +
            'class="scribe-plugin-link-tooltip-show-on-view"></a>' +
            '<input data-scribe-plugin-link-tooltip-role="input"' +
            'class="scribe-plugin-link-tooltip-show-on-edit" placeholder="' + translationsDialog.pasteOrTypeLink + '"/>' +
            '<button data-scribe-plugin-link-tooltip-role="submit" type="submit"' +
            'class="scribe-plugin-link-tooltip-show-on-edit"><span class="fa fa-check"></span></button>' +
            '<button data-scribe-plugin-link-tooltip-role="edit" type="button"' +
            'class="scribe-plugin-link-tooltip-show-on-view"><span class="fa fa-pencil"></span></button>' +
            '<button data-scribe-plugin-link-tooltip-role="remove" type="button"' +
            'class="scribe-plugin-link-tooltip-show-on-view"><span class="fa fa-trash"></span></button>';

        scribe.use(scribePluginLinkTooltip({
            innerMarkup: scribePluginLinkTooltipTemplate,
            namespace: 'scribe-plugin-link-tooltip',
            linkSanitizer: function (str) {
                // Try to catch common cases of users forgetting to add "http://" in front,
                // but err on the safe side: if it looks even remotely like a hostname, just stop.
                // Feel free to add your favourite contry's TLD as long as it is not a common file extension.
                return str.match(/^\w[\w\-_\.]+\.(co|uk|com|org|net|gov|biz|info|us|eu|de|fr|it|es|pl|nz)/i) ?
                'http://' + str :
                    str;
            }
        }));

        scribe.use(scribePluginIntelligentUnlinkCommand());

        scribe.use(scribePluginFormatterHtmlEnsureSemanticElements());

        return scribe;
    };
});