package DE_EPAGES::MultiStoreConnector::t::ProductTypeAttributeSynchronization;
use base DE_EPAGES::Database::API::Test;

use strict;
use Test::More tests => 86;

use DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization;
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByGUID LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

#global variables
our $TestShopAlias = 'MSIntegrationTestShop';
our $TestShopPath = '/Shops/' . 'MSIntegrationTestShop';
our $TestProductTypeAlias = 'IntegrationProductType';
our $TestProductTypePath = $TestShopPath . '/ProductTypes/' . $TestProductTypeAlias;
our $TestShopXML = 'MSIntegrationTestShop.xml';
our $SQLInjection = "SET foreign_key_checks = 0; DROP TABLE object;";
our $hTestProductTypeAttributeSourceArrayPosition = {
    'MSIntegrationTestProductTypeAttribute-01' => 0,
    'MSIntegrationTestProductTypeAttribute-02' => 1,
    $SQLInjection => 0,
    'MSIntegrationTestInjectionProductTypeAttribute-02' => 1,
};

our $hTestProductTypeAttributeOptionSourceArrayPosition = {
    'Red' => 0,
    'Blue' => 1,
    $SQLInjection => 0,
};

sub test_parameter_type {
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization->new();
    
    #info of attributes and their expected data types
    my $hAttributeParameters = {
        'alias' => 'string',
        'type' => 'string',
        'position' => 'integer',
        'is_visible' => 'boolean',
        'is_html' => 'boolean',
        'show_in_own_tab' => 'boolean',
        'is_search_filter' => 'boolean',
        'options' => 'array',
    };

    foreach my $AttributeName (keys %$hAttributeParameters){
        is($Synchronizer->getAttributeParameterType($AttributeName), $hAttributeParameters->{$AttributeName} , 'parameter type attribute \'' . $AttributeName . '\' correct')
    }
      
}

sub test_mandatory {

    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization->new();

    #info of attributes mandatory
    my $hAttributeMandatory = {
        # object
        'alias' => 1,
        'position' => 1,
        
        # attribute
        'type' => 1,
        'length' => 0,
        'is_visible' => 1,
        'is_html' => 1,
        'show_in_own_tab' => 0,
        'is_search_filter' => 0,

        # predefattribute settings
        'options' => 0,
    };
    
    foreach my $AttributeName (keys %$hAttributeMandatory){
        is($Synchronizer->getAttributeMandatory($AttributeName), $hAttributeMandatory->{$AttributeName} , 'mandatory attribute \'' . $AttributeName . '\' correct')
    }
}

sub test_producttype_attributes{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization->new();
    my $TestProductTypeGUID = LoadObjectByPath($TestProductTypePath)->get('GUID');
    
    #get test data
    my $aTestProductTypeAttributes = [{
        'alias' => 'MSIntegrationTestProductTypeAttribute-01',
        'type' => 'String',
        'length' => 255,
        'position' => 17,
        'is_visible' => 'true',
        'is_html' => 'true',
        'show_in_own_tab' => 'true',
        'product_type' => $TestProductTypeGUID,
    },
    {
        'alias' => 'MSIntegrationTestProductTypeAttribute-02',
        'type' => 'PreDefString',
        'position' => 10,
        'is_visible' => 'true',
        'is_html' => 'true',
        'is_search_filter' => 'true',
        'product_type' => $TestProductTypeGUID,
        'options' => [
            {
                'alias' => 'Red',
                'position' => 10,
            },
            {
                'alias' => 'Blue',
                'position' => 20,
            },
        ]
    }];

    
    #####TEST PRODUCT TYPES#####
    foreach my $hProductTypeAttribute (@$aTestProductTypeAttributes){
    
        #check if test product types not exists
        ok(not (ExistsObjectByPath($TestProductTypePath . '/Attributes/' . $hProductTypeAttribute->{'alias'})), 'test product type "' . $hProductTypeAttribute->{'alias'} . '" not exists');
    }
    #add product type attributes
    # as the API allows to add attributes for different product types, we need to filter the result by the GUID of the test product type
    my $aProductTypeAttributesResult = $Synchronizer->addProductTypeAttributes($TestShopAlias, $aTestProductTypeAttributes)->{$TestProductTypeGUID};
    my $hTestProductTypeAttributeReturnArrayPosition;
    my $hTestProductTypeAttributeOptionReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    my $OptionCounter = -1;

    ok(defined($aProductTypeAttributesResult), 'product type attribute synchronization provides a result');
    

    #remember positions and guids
    foreach my $hResultProductTypeAttribute (@$aProductTypeAttributesResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProductTypeAttribute->{'alias'};
        my $ResultGUID = $hResultProductTypeAttribute->{'guid'};
        
        #remember position; this does not work if different attributes have the same alias
        $hTestProductTypeAttributeReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);

        # remember position of attribute options (predefattributes)
        my $aResultProductTypeAttributeOptions = $hResultProductTypeAttribute->{'options'};
        if (defined($aResultProductTypeAttributeOptions)) {
            foreach my $hResultProductTypeOption (@$aResultProductTypeAttributeOptions) {
                $OptionCounter++;

                $ResultAlias = $hResultProductTypeOption->{'alias'};

                #remember position; this does not work if different options have the same alias
                $hTestProductTypeAttributeOptionReturnArrayPosition->{$ResultAlias} = $OptionCounter;
            }
        }
        
    }
    
    #start testing
    foreach my $hProductTypeAttribute (@$aTestProductTypeAttributes){
    
        #get positions
        my $ResultPosition = $hTestProductTypeAttributeReturnArrayPosition->{$hProductTypeAttribute->{'alias'}};
        my $SourcePosition = $hTestProductTypeAttributeSourceArrayPosition->{$hProductTypeAttribute->{'alias'}};
    
        #get result info
        my $ResultGUID = $aProductTypeAttributesResult->[$ResultPosition]->{'guid'};
        my $ResultAlias = $aProductTypeAttributesResult->[$ResultPosition]->{'alias'};                                
        my $ResultEP6ID = $aProductTypeAttributesResult->[$ResultPosition]->{'epages6-objectid'};
        
        ok(((defined $ResultGUID) and ($ResultGUID ne '')),'guid exists and is not undef');
        is($ResultAlias, $hProductTypeAttribute->{'alias'},'alias "' . $hProductTypeAttribute->{'alias'} . '" is correct');
        ok(((defined $ResultEP6ID) and ($ResultEP6ID ne '')),'epages6 id exists and is not undef');
        
        #get object from database
        my $AddedTestObject = LoadObject($ResultEP6ID);
        my $AddedTestObjectByGUID = LoadObjectByGUID($ResultGUID);
        
        #check content with epages6 api
        is($AddedTestObject->id(), $AddedTestObjectByGUID->id(),"GUID is correct, epages 6 object api");
        is($AddedTestObject->get('Position'), $aTestProductTypeAttributes->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
        is($AddedTestObject->alias(), $aTestProductTypeAttributes->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsVisible'), $aTestProductTypeAttributes->[$SourcePosition]->{'is_visible'}, 'is_visible from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsHTML'), $aTestProductTypeAttributes->[$SourcePosition]->{'is_html'}, 'is_html from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsSearchFilter'), $aTestProductTypeAttributes->[$SourcePosition]->{'is_search_filter'}, 'is_search_filter from db is correct, epages 6 object api') if defined($aTestProductTypeAttributes->[$SourcePosition]->{'is_search_filter'});
        is($AddedTestObject->get('ShowInOwnTab'), $aTestProductTypeAttributes->[$SourcePosition]->{'show_in_own_tab'}, 'show_in_own_tab from db is correct, epages 6 object api') if defined($aTestProductTypeAttributes->[$SourcePosition]->{'show_in_own_tab'});

        # check if attribute has options (predefattributes) 
        my $aProductTypeAttributeOptions = $hProductTypeAttribute->{'options'};
        if(defined($aProductTypeAttributeOptions)) {
            # verify that number of options is correct
            is(scalar(@$aProductTypeAttributeOptions), scalar(@{$AddedTestObject->child('PreDefAttributes')->children}), 'number of predefattributes correct, epages 6 object api');

            foreach my $hProductTypeAttributeOption (@$aProductTypeAttributeOptions) {

                my $OptionResultPosition = $hTestProductTypeAttributeOptionReturnArrayPosition->{$hProductTypeAttributeOption->{'alias'}};
                my $OptionSourcePosition = $hTestProductTypeAttributeOptionSourceArrayPosition->{$hProductTypeAttributeOption->{'alias'}};

                #get result info
                my $OptionResultGUID = $aProductTypeAttributesResult->[$ResultPosition]->{'options'}->[$OptionResultPosition]->{'guid'};
                my $OptionResultAlias = $aProductTypeAttributesResult->[$ResultPosition]->{'options'}->[$OptionResultPosition]->{'alias'};                                
                my $OptionResultEP6ID = $aProductTypeAttributesResult->[$ResultPosition]->{'options'}->[$OptionResultPosition]->{'epages6-objectid'};

                ok(((defined $OptionResultGUID) and ($OptionResultGUID ne '')),'guid exists and is not undef');
                is($OptionResultAlias, $hProductTypeAttributeOption->{'alias'},'alias "' . $hProductTypeAttributeOption->{'alias'} . '" is correct');
                ok(((defined $OptionResultEP6ID) and ($OptionResultEP6ID ne '')),'epages6 id exists and is not undef');

                #get object from database
                my $AddedTestObjectOption = LoadObject($OptionResultEP6ID);
                my $AddedTestObjectOptionByGUID = LoadObjectByGUID($OptionResultGUID);

                #check content with epages6 api
                is($AddedTestObjectOption->id(), $AddedTestObjectOptionByGUID->id(),"GUID is correct, epages 6 object api");
                is($AddedTestObjectOption->get('Position'), $aTestProductTypeAttributes->[$SourcePosition]->{'options'}->[$OptionSourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
                is($AddedTestObjectOption->alias(), $aTestProductTypeAttributes->[$SourcePosition]->{'options'}->[$OptionSourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
            }
        }
    }
}

sub test_sql_injection{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization->new();
    my $TestProductTypeGUID = LoadObjectByPath($TestProductTypePath)->get('GUID');
    
    #get test data
    my $aInjectionTestProductTypeAttributes = [{
        'alias' => $SQLInjection,
        'type' => 'String',
        'length' => 255,
        'position' => 17,
        'is_visible' => 'true',
        'is_html' => 'true',
        'show_in_own_tab' => 'true',
        'product_type' => $TestProductTypeGUID,
    },
    {
        'alias' => 'MSIntegrationTestInjectionProductTypeAttribute-02',
        'type' => 'PreDefString',
        'position' => 10,
        'is_visible' => 'true',
        'is_html' => 'true',
        'is_search_filter' => 'true',
        'product_type' => $TestProductTypeGUID,
        'options' => [
            {
                'alias' => $SQLInjection,
                'position' => 10,
            },
            {
                'alias' => 'Blue',
                'position' => 20,
            },
        ]
    }];

    
    #####TEST PRODUCT TYPES#####
    foreach my $hProductTypeAttribute (@$aInjectionTestProductTypeAttributes){
    
        #check if test product types not exists
        ok(not (ExistsObjectByPath($TestProductTypePath . '/Attributes/' . $hProductTypeAttribute->{'alias'})), 'test product type "' . $hProductTypeAttribute->{'alias'} . '" not exists');
    }
    #add product type attributes
    # as the API allows to add attributes for different product types, we need to filter the result by the GUID of the test product type
    my $aProductTypeAttributesResult = $Synchronizer->addProductTypeAttributes($TestShopAlias, $aInjectionTestProductTypeAttributes)->{$TestProductTypeGUID};
    my $hTestProductTypeAttributeReturnArrayPosition;
    my $hTestProductTypeAttributeOptionReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    my $OptionCounter = -1;

    ok(defined($aProductTypeAttributesResult), 'product type attribute synchronization provides a result');
    

    #remember positions and guids
    foreach my $hResultProductTypeAttribute (@$aProductTypeAttributesResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProductTypeAttribute->{'alias'};
        my $ResultGUID = $hResultProductTypeAttribute->{'guid'};
        
        #remember position; this does not work if different attributes have the same alias
        $hTestProductTypeAttributeReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);

        # remember position of attribute options (predefattributes)
        my $aResultProductTypeAttributeOptions = $hResultProductTypeAttribute->{'options'};
        if (defined($aResultProductTypeAttributeOptions)) {
            foreach my $hResultProductTypeOption (@$aResultProductTypeAttributeOptions) {
                $OptionCounter++;

                $ResultAlias = $hResultProductTypeOption->{'alias'};

                #remember position; this does not work if different options have the same alias
                $hTestProductTypeAttributeOptionReturnArrayPosition->{$ResultAlias} = $OptionCounter;
            }
        }
        
    }
    
    #start testing
    foreach my $hProductTypeAttribute (@$aInjectionTestProductTypeAttributes){
    
        #get positions
        my $ResultPosition = $hTestProductTypeAttributeReturnArrayPosition->{$hProductTypeAttribute->{'alias'}};
        my $SourcePosition = $hTestProductTypeAttributeSourceArrayPosition->{$hProductTypeAttribute->{'alias'}};
    
        #get result info
        my $ResultGUID = $aProductTypeAttributesResult->[$ResultPosition]->{'guid'};
        my $ResultAlias = $aProductTypeAttributesResult->[$ResultPosition]->{'alias'};                                
        my $ResultEP6ID = $aProductTypeAttributesResult->[$ResultPosition]->{'epages6-objectid'};
        
        ok(((defined $ResultGUID) and ($ResultGUID ne '')),'guid exists and is not undef');
        is($ResultAlias, $hProductTypeAttribute->{'alias'},'alias "' . $hProductTypeAttribute->{'alias'} . '" is correct');
        ok(((defined $ResultEP6ID) and ($ResultEP6ID ne '')),'epages6 id exists and is not undef');
        
        #get object from database
        my $AddedTestObject = LoadObject($ResultEP6ID);
        my $AddedTestObjectByGUID = LoadObjectByGUID($ResultGUID);
        
        #check content with epages6 api
        is($AddedTestObject->id(), $AddedTestObjectByGUID->id(),"GUID is correct, epages 6 object api");
        is($AddedTestObject->get('Position'), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
        is($AddedTestObject->alias(), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsVisible'), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'is_visible'}, 'is_visible from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsHTML'), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'is_html'}, 'is_html from db is correct, epages 6 object api');
        is($AddedTestObject->get('IsSearchFilter'), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'is_search_filter'}, 'is_search_filter from db is correct, epages 6 object api') 
            if defined($aInjectionTestProductTypeAttributes->[$SourcePosition]->{'is_search_filter'});
        is($AddedTestObject->get('ShowInOwnTab'), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'show_in_own_tab'}, 'show_in_own_tab from db is correct, epages 6 object api') 
            if defined($aInjectionTestProductTypeAttributes->[$SourcePosition]->{'show_in_own_tab'});

        # check if attribute has options (predefattributes) 
        my $aProductTypeAttributeOptions = $hProductTypeAttribute->{'options'};
        if(defined($aProductTypeAttributeOptions)) {
            # verify that number of options is correct
            is(scalar(@$aProductTypeAttributeOptions), scalar(@{$AddedTestObject->child('PreDefAttributes')->children}), 'number of predefattributes correct, epages 6 object api');

            foreach my $hProductTypeAttributeOption (@$aProductTypeAttributeOptions) {

                my $OptionResultPosition = $hTestProductTypeAttributeOptionReturnArrayPosition->{$hProductTypeAttributeOption->{'alias'}};
                my $OptionSourcePosition = $hTestProductTypeAttributeOptionSourceArrayPosition->{$hProductTypeAttributeOption->{'alias'}};

                #get result info
                my $OptionResultGUID = $aProductTypeAttributesResult->[$ResultPosition]->{'options'}->[$OptionResultPosition]->{'guid'};
                my $OptionResultAlias = $aProductTypeAttributesResult->[$ResultPosition]->{'options'}->[$OptionResultPosition]->{'alias'};                                
                my $OptionResultEP6ID = $aProductTypeAttributesResult->[$ResultPosition]->{'options'}->[$OptionResultPosition]->{'epages6-objectid'};

                ok(((defined $OptionResultGUID) and ($OptionResultGUID ne '')),'guid exists and is not undef');
                is($OptionResultAlias, $hProductTypeAttributeOption->{'alias'},'alias "' . $hProductTypeAttributeOption->{'alias'} . '" is correct');
                ok(((defined $OptionResultEP6ID) and ($OptionResultEP6ID ne '')),'epages6 id exists and is not undef');

                #get object from database
                my $AddedTestObjectOption = LoadObject($OptionResultEP6ID);
                my $AddedTestObjectOptionByGUID = LoadObjectByGUID($OptionResultGUID);

                #check content with epages6 api
                is($AddedTestObjectOption->id(), $AddedTestObjectOptionByGUID->id(),"GUID is correct, epages 6 object api");
                is($AddedTestObjectOption->get('Position'), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'options'}->[$OptionSourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
                is($AddedTestObjectOption->alias(), $aInjectionTestProductTypeAttributes->[$SourcePosition]->{'options'}->[$OptionSourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
            }
        }
    }
}

sub test {
    
    #test attributes parameter types function
    test_parameter_type();
    
    #test attributes mandatory function
    test_mandatory();
    
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            
            #check if shop exists
            ok(ExistsObjectByPath($TestShopPath), 'integration test shop exists');
            
            #test normal product type attributes
            test_producttype_attributes();
            
            #test sql injections
            test_sql_injection();
        }
    );
}


sub cleanup {

    #delete integration test shop if exists
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            LoadObjectByPath($TestShopPath)->delete if ExistsObjectByPath($TestShopPath);
        }
    );

    return 1;
}

sub init {

    #add integration test store
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            ImportObjects( $TestShopXML, '/Shops' );
        }
    );

    return 1;
}

__PACKAGE__->runScript;