package DE_EPAGES::MultiStoreConnector::t::ProductTypeSynchronization;
use base DE_EPAGES::Database::API::Test;

use strict;
use Test::More tests => 59;

use DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization;
use DE_EPAGES::Core::API::Error qw (GetError ExistsError);
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByGUID LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

#global variables
our $TestShopAlias = 'MSIntegrationTestShop';
our $TestShopPath = '/Shops/' . 'MSIntegrationTestShop';
our $TestShopXML = 'MSIntegrationTestShop.xml';
our $hTestProductTypeSourceArrayPosition = {
    'MSIntegrationTestProductType-01' => 0,
    'MSIntegrationTestProductType-02' => 1,
    'MSIntegrationTestProductTypeNoAttributes-01' => 0,
    'MSIntegrationTestProductTypeNoAttributes-02' => 1,
};

sub test_parameter_type {
    
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();
    
    #info of attributes and their expected data types
    my $hAttributeParameters = {
        # object
        'alias' => 'string',
        'guid' => 'string',
        'position' => 'integer',
        
        # producttype
        'customizeisfontsizeselectable' => 'boolean',
        'customizefontsize' => 'integer',
    };

    foreach my $AttributeName (keys %$hAttributeParameters){
        is($Synchronizer->getAttributeParameterType($AttributeName), $hAttributeParameters->{$AttributeName} , 'parameter type attribute \'' . $AttributeName . '\' correct')
    }
      
}

sub test_mandatory {

    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();

    #info of attributes mandatory
    my $hAttributeMandatory = {
        # object
        'alias' => 1,
        'guid' => 0,
        'position' => 1,

        # producttype
        'customizeisfontsizeselectable' => 0,
        'customizefontsize' => 0,
    };
    
    foreach my $AttributeName (keys %$hAttributeMandatory){
        is($Synchronizer->getAttributeMandatory($AttributeName), $hAttributeMandatory->{$AttributeName} , 'mandatory attribute \'' . $AttributeName . '\' correct')
    }
}

sub test_producttypes{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();
    
    #get test data
    my $aTestProductTypes = [{
        'alias' => 'MSIntegrationTestProductType-01',
        'position' => 17,
        'customizeisfontsizeselectable' => 'true',
        'customizefontsize' => 12,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList',
        },
        'attributes' => [
            {
                'alias' => 'Material',
                'type' => 'String',
                'length' => 255,
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            },
            {
                'alias' => 'Color',
                'type' => 'PreDefString',
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
                'options' => [
                    {
                        'alias' => 'Red',
                        'position' => 10,
                    },
                    {
                        'alias' => 'Blue',
                        'position' => 20,
                    },
                ]
            }
        ]
    },
    {
        'alias' => 'MSIntegrationTestProductType-02',
        'position' => 17,
        'customizeisfontsizeselectable' => 'false',
        'customizefontsize' => 14,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList',
        },
        'attributes' => [
            {
                'alias' => 'Material',
                'type' => 'String',
                'length' => 255,
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            },
            {
                'alias' => 'Color',
                'type' => 'PreDefString',
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
                'options' => [
                    {
                        'alias' => 'Red',
                        'position' => 10,
                    },
                    {
                        'alias' => 'Blue',
                        'position' => 20,
                    },
                ]
            }
        ]
    }
    ];
    
    #####TEST PRODUCT TYPES#####
    foreach my $hProductType (@$aTestProductTypes){
    
        #check if test product types not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/ProductTypes/' . $hProductType->{'alias'})), 'test product type "' . $hProductType->{'alias'} . '" not exists');
    }
    #add products
    my $aProductTypesResult = $Synchronizer->addProductTypesToShop($TestShopAlias, $aTestProductTypes);
    my $hTestProductTypeReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    
    #remember positions and guids
    foreach my $hResultProductType (@$aProductTypesResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProductType->{'alias'};
        my $ResultGUID = $hResultProductType->{'guid'};
        
        #remember position
        $hTestProductTypeReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);
        
    }
    
    #start testing
    foreach my $hProductType (@$aTestProductTypes){
    
        #get positions
        my $ResultPosition = $hTestProductTypeReturnArrayPosition->{$hProductType->{'alias'}};
        my $SourcePosition = $hTestProductTypeSourceArrayPosition->{$hProductType->{'alias'}};
    
        #get result info
        my $ResultGUID = $aProductTypesResult->[$ResultPosition]->{'guid'};
        my $ResultAlias = $aProductTypesResult->[$ResultPosition]->{'alias'};                                
        my $ResultEP6ID = $aProductTypesResult->[$ResultPosition]->{'epages6-objectid'};
        
        ok(((defined $ResultGUID) and ($ResultGUID ne '')),'guid exists and is not undef');
        is($ResultAlias, $hProductType->{'alias'},'alias "' . $hProductType->{'alias'} . '" is correct');
        ok(((defined $ResultEP6ID) and ($ResultEP6ID ne '')),'epages6 id exists and is not undef');
        
        #get object from database
        my $AddedTestObject = LoadObject($ResultEP6ID);
        my $AddedTestObjectByGUID = LoadObjectByGUID($ResultGUID);
        
        #check content with epages6 api
        is($AddedTestObject->id(), $AddedTestObjectByGUID->id(),"GUID is correct, epages 6 object api");
        is($AddedTestObject->get('Position'), $aTestProductTypes->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
        is($AddedTestObject->alias(), $aTestProductTypes->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
        #is($AddedTestObject->get('CustomizeIsFontSizeSelectable'), $aTestProductTypes->[$SourcePosition]->{'customizeisfontsizeselectable'}, 'CustomizeIsFontSizeSelectable from db is correct, epages 6 object api');
        #is($AddedTestObject->get('CustomizeFontSize'), $aTestProductTypes->[$SourcePosition]->{'customizefontsize'}, 'customize font size from db is correct, epages 6 object api');

        #verify attributes
        my $ahExpectedProductTypeAttributes = $hProductType->{'attributes'};
        my $aSavedProductTypeAttributes = $AddedTestObject->child('Attributes')->children;

        # verify number of attributes for the product type
        is(scalar(@$ahExpectedProductTypeAttributes), scalar(@$aSavedProductTypeAttributes), 'number of attributes correct, epages 6 object api');

        foreach my $hProductTypeAttribute (@$ahExpectedProductTypeAttributes) {
            my $AttributeAlias = $hProductTypeAttribute->{'alias'};
            my $SavedAttribute = $AddedTestObject->child('Attributes')->child($AttributeAlias);

            # check that every attribute is defined
            ok(defined($SavedAttribute), 'attribute is defined, epages 6 object api');

            # check if attribute has options (predefattributes) 
            if(defined($hProductTypeAttribute->{'options'})) {
                my $ahExpectedAttributeOptions = $hProductTypeAttribute->{'options'};
                my $aSavedAttributeOptions = $SavedAttribute->child('PreDefAttributes')->children;

                # verify that number of options is correct
                is(scalar(@$ahExpectedAttributeOptions), scalar(@$aSavedAttributeOptions), 'number of predefattributes correct, epages 6 object api');

                foreach my $hAttributeOption (@$ahExpectedAttributeOptions) {
                    my $OptionAlias = $hAttributeOption->{'alias'};
                    my $SavedAttributeOption = $SavedAttribute->child('PreDefAttributes')->child($OptionAlias);
                    # verify that option is defined
                    ok(defined($SavedAttributeOption), 'predefattribute is defined, epages 6 object api');

                }
            }
        }
    }
}

sub test_producttypes_without_attributes{
    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();
    
    #get test data
    my $aTestProductTypes = [{
        'alias' => 'MSIntegrationTestProductTypeNoAttributes-01',
        'position' => 17,
        'customizeisfontsizeselectable' => 'true',
        'customizefontsize' => 12,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList',
        }
    },
    {
        'alias' => 'MSIntegrationTestProductTypeNoAttributes-02',
        'position' => 17,
        'customizeisfontsizeselectable' => 'false',
        'customizefontsize' => 14,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList',
        }
    }
    ];
    
    #####TEST PRODUCT TYPES#####
    foreach my $hProductType (@$aTestProductTypes){
    
        #check if test product types not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/ProductTypes/' . $hProductType->{'alias'})), 'test product type "' . $hProductType->{'alias'} . '" not exists');
    }
    #add products
    my $aProductTypesResult = $Synchronizer->addProductTypesToShop($TestShopAlias, $aTestProductTypes);
    my $hTestProductTypeReturnArrayPosition;
    my $aReturnGUIDs;
    my $Counter = -1;
    
    #remember positions and guids
    foreach my $hResultProductType (@$aProductTypesResult){
        #increase counter
        $Counter += 1;
        
        #get info
        my $ResultAlias = $hResultProductType->{'alias'};
        my $ResultGUID = $hResultProductType->{'guid'};
        
        #remember position
        $hTestProductTypeReturnArrayPosition->{$ResultAlias} = $Counter;
        
        #remember guids
        push(@$aReturnGUIDs, $ResultGUID);
        
    }
    
    #start testing
    foreach my $hProductType (@$aTestProductTypes){
    
        #get positions
        my $ResultPosition = $hTestProductTypeReturnArrayPosition->{$hProductType->{'alias'}};
        my $SourcePosition = $hTestProductTypeSourceArrayPosition->{$hProductType->{'alias'}};
    
        #get result info
        my $ResultGUID = $aProductTypesResult->[$ResultPosition]->{'guid'};
        my $ResultAlias = $aProductTypesResult->[$ResultPosition]->{'alias'};                                
        my $ResultEP6ID = $aProductTypesResult->[$ResultPosition]->{'epages6-objectid'};
        
        ok(((defined $ResultGUID) and ($ResultGUID ne '')),'guid exists and is not undef');
        is($ResultAlias, $hProductType->{'alias'},'alias "' . $hProductType->{'alias'} . '" is correct');
        ok(((defined $ResultEP6ID) and ($ResultEP6ID ne '')),'epages6 id exists and is not undef');
        
        #get object from database
        my $AddedTestObject = LoadObject($ResultEP6ID);
        my $AddedTestObjectByGUID = LoadObjectByGUID($ResultGUID);
        
        #check content with epages6 api
        is($AddedTestObject->id(), $AddedTestObjectByGUID->id(),"GUID is correct, epages 6 object api");
        is($AddedTestObject->get('Position'), $aTestProductTypes->[$SourcePosition]->{'position'}, 'position from db is correct, epages 6 object api');
        is($AddedTestObject->alias(), $aTestProductTypes->[$SourcePosition]->{'alias'}, 'alias from db is correct, epages 6 object api');
        #is($AddedTestObject->get('CustomizeIsFontSizeSelectable'), $aTestProductTypes->[$SourcePosition]->{'customizeisfontsizeselectable'}, 'CustomizeIsFontSizeSelectable from db is correct, epages 6 object api');
        #is($AddedTestObject->get('CustomizeFontSize'), $aTestProductTypes->[$SourcePosition]->{'customizefontsize'}, 'customize font size from db is correct, epages 6 object api');   
    }
}

sub test_producttypes_wrong_layout {
     #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();
    
    #get test data
    my $aTestProductTypes = [{
        'alias' => 'MSIntegrationTestProductTypeWrongLayout-01',
        'position' => 17,
        'customizeisfontsizeselectable' => 'true',
        'customizefontsize' => 12,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList',
        },
        'attributes' => [
            {
                'alias' => 'Material',
                'type' => 'String',
                'length' => 255,
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            },
            {
                'alias' => 'Size',
                'type' => 'Integer',
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            }
        ]
    },
    {
        'alias' => 'MSIntegrationTestProductTypeWrongLayout-02',
        'position' => 17,
        'customizeisfontsizeselectable' => 'false',
        'customizefontsize' => 14,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList-42',
        },
        'attributes' => [
            {
                'alias' => 'Material',
                'type' => 'String',
                'length' => 255,
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            },
            {
                'alias' => 'Size',
                'type' => 'Integer',
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            }
        ]
    }
    ];
    
    #####TEST PRODUCT TYPES#####
    foreach my $hProductType (@$aTestProductTypes){
    
        #check if test product types not exists
        ok(not (ExistsObjectByPath($TestShopPath . '/ProductTypes/' . $hProductType->{'alias'})), 'test product type "' . $hProductType->{'alias'} . '" not exists');
    }
    #add product types
    eval {
        # this should throw an error
        $Synchronizer->addProductTypesToShop($TestShopAlias, $aTestProductTypes);
    };

    ok(ExistsError, 'Error raised correctly');

    my $MyError = GetError();

    is($MyError->code, 'TemplateTypeNotFound', 'TemplateTypeNotFound error raised correctly');

    foreach my $hProductType (@$aTestProductTypes){

        #verify that product type was not created
        ok(not (ExistsObjectByPath($TestShopPath . '/ProductTypes/' . $hProductType->{'alias'})), 'test product type "' . $hProductType->{'alias'} . '" not exists');
    }
}

sub test_sql_injection {

    #get variables
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();
    
    #get test data
    my $SQLInjection = "SET foreign_key_checks = 0; DROP TABLE object;";
    my $aInjectionTestProductTypes = [{
        'alias' => $SQLInjection,
        'position' => 0,
        'customizeisfontsizeselectable' => 'true',
        'customizefontsize' => 12,
        'layout' => {
            'Content-Base' => 'ImageLeft_New',
            'Content-Variation' => 'VariationSelectBox',
            'Content-Bundles' => 'BundlesCompactList',
        },
        'attributes' => [
            {
                'alias' => 'Material',
                'type' => 'String',
                'length' => 255,
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            },
            {
                'alias' => 'Size',
                'type' => 'Integer',
                'is_visible' => 'true',
                'is_html' => 'true',
                'position' => 10,
                'show_in_own_tab' => 'false',
                'is_search_filter' => 'false',
            }
        ]
    }];
    
    #test if injection product types not exists
    foreach my $hInjectionProductType (@$aInjectionTestProductTypes){
    
        ok(not (ExistsObjectByPath($TestShopPath . '/ProductTypes/' . $hInjectionProductType->{'alias'})), 'test product type "' . $hInjectionProductType->{'alias'} . '" not exists');
    }
    
    #add injection product types
    my $aInjectionProductTypesResult = $Synchronizer->addProductTypesToShop($TestShopAlias, $aInjectionTestProductTypes);
    
    
    foreach my $hInjectionProductType (@$aInjectionProductTypesResult){
        
        #get positions
        my $ResultPosition = 0;
    
        #get result info                              
        my $ResultEP6ID = $aInjectionProductTypesResult->[$ResultPosition]->{'epages6-objectid'};
        
        my $AddedInjectionTestObject = LoadObject($ResultEP6ID);
        
        #check if injection test product types exists
        is($AddedInjectionTestObject->alias , $hInjectionProductType->{'alias'}, 'injection test product type "' . $hInjectionProductType->{'alias'} . '" exists, database survived');
    }
    
    # TODO test GET
}

sub test {
    
    #test attributes parameter types function
    test_parameter_type();
    
    #test attributes mandatory function
    test_mandatory();
    
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            
            #check if shop exists
            ok(ExistsObjectByPath($TestShopPath), 'integration test shop exists');
            
            #test normal product types
            test_producttypes();

            test_producttypes_without_attributes();
            
            #test non product types
            #test_non_product_type_guids();
            
            #test sql injections
            test_sql_injection();

            test_producttypes_wrong_layout();

        }
    );
}


sub cleanup {

    #delete integration test shop if exists
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            LoadObjectByPath($TestShopPath)->delete if ExistsObjectByPath($TestShopPath);
        }
    );

    return 1;
}

sub init {

    #add integration test store
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            ImportObjects( $TestShopXML, '/Shops' );
        }
    );

    return 1;
}

__PACKAGE__->runScript;