#========================================================================================
# package      DE_EPAGES::Product::t::unit::WatchedProductsService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::WatchedProductsService
#========================================================================================
package DE_EPAGES::Product::t::unit::WatchedProductsService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::WatchedProductsService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_getNumberOfWatchersPerProduct();
    $self->test_getWatchedProductDetails();

    done_testing();
}




sub test_getNumberOfWatchersPerProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $WatchedProductsService = $Mockery->get('SUT');

    my $hWatchCount = $WatchedProductsService->getNumberOfWatchersPerProduct(5678);

    my $hExpectedWatchCount = {
        '31415' => 1,
        '1234' => 2
    };

    is_deeply($hWatchCount, $hExpectedWatchCount, "$SubTestName - test if expected WatchCount was returned.");
}



sub test_getWatchedProductDetails {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();
    my $WatchedProductsService = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $hProductDetails = $WatchedProductsService->getWatchedProductDetails($Shop, 'de_DE', 12345, 5); ## no critic (ProhibitMagicNumbers)

    my $hExpectedProductDetails = {
        'watchers' => 5,
        'name' => 'TestProduct',
        'productNumber' => 'Product-123',
        'productId' => '1234-546754-324356'
    };

    is_deeply($hProductDetails, $hExpectedProductDetails, "$SubTestName - test if expected ProductDetails were returned.");
}



sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['ProductClass', 'Locale'], sub {
        my ($ProductClass, $Locale) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Product::API::WatchedProductsService', [$ProductClass]);
        $Mockify->addMockWithReturnValueAndParameterCheck('_DAL_GetWatchedProducts', [[1234, 2], [31415, 1]], [{'int' => 5678}]);
        $Mockify->addMockWithReturnValueAndParameterCheck('_loadLocale', $Locale, [{'string' => 'de_DE'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('ProductClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Class', {});
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', ['Product'], sub {
        my ($Product) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'ID' => 5678,
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('loadObject', $Product, [{'int' => '12345'}, 'object']);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Product', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'Alias' => 'Product-123',
            'Name' => 'TestProduct',
            'GUID' => '1234-546754-324356',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('Locale', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Locale', {
            'LanguageID' => 1,
        });
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}

__PACKAGE__->runScript();