﻿use Test::More;
use DE_EPAGES::ShopConfiguration::API::Object::RemoteObject;
use DE_EPAGES::Database::API::Connection qw (RunOnStore GetTestSite);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::WebService::API::WebService::BaseService;

use strict;
use utf8;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            my $RemoteObject = DE_EPAGES::ShopConfiguration::API::Object::RemoteObject->new(
                'Store' => LoadRootObject->get('Stores')->[0],
                'ObjectID' => '1');

            _TestNormalString($RemoteObject);
            _TestUTF8String($RemoteObject);
            _TestEmptyArray($RemoteObject);
            _TestNormalArray($RemoteObject);
            _TestUTF8Array($RemoteObject);
            _TestComplexArray($RemoteObject);

            done_testing();
        }
    );
}

sub _TestEmptyArray {
    my ($RemoteObject) = @_;

    my $aValues = $RemoteObject->formatUTF8Values([]);

    is(scalar(@{$aValues}),0,'empty array');
}

sub _TestComplexArray {
    my ($RemoteObject) = @_;

    my $aValues = $RemoteObject->formatUTF8Values([{'test' => 'test123'},'blub2']);

    is(ref($aValues->[0]),'HASH','complex simple array');
    is($aValues->[1],'blub2','complex simple array');
}
sub _TestNormalArray {
    my ($RemoteObject) = @_;

    my $aValues = $RemoteObject->formatUTF8Values(['blub','blub2']);

    is($aValues->[0],'blub','normal simple array');
    is($aValues->[1],'blub2','normal simple array');
}

sub _TestUTF8Array {
    my ($RemoteObject) = @_;

    my $aValues = $RemoteObject->formatUTF8Values(['blüb','blüb2']);

    is($aValues->[0]->isa('SOAP::Data'),1,'utf8 simple array');
    is($aValues->[1]->isa('SOAP::Data'),1,'utf8 simple array');
    is($aValues->[0]->value(),'blüb','utf8 simple array');
    is($aValues->[1]->value(),'blüb2','utf8 simple array');
}

sub _TestNormalString {
    my ($RemoteObject) = @_;

    my $Value = $RemoteObject->formatUTF8Values('Test123');

    is($Value,'Test123','normal string');
}

sub _TestUTF8String {
    my ($RemoteObject) = @_;

    my $Value = $RemoteObject->formatUTF8Values('blüb');
    is($Value->isa('SOAP::Data'),1,'utf8 string');
    is($Value->value(),'blüb','utf8 string');
}


RunScript( Sub => \&Main );

1;
